/**************************************************************************\
*   BC++ 1.01                                   Source: READCFG.C
*   Host platform:   DOS                        Code:   ASCII-7
*   Target platform: DOS                        Code:   ASCII-7
*   Project name:    PL-Mail: Local FTN-Mailer (BSO)
*   Last change:     30.11.97
*   Programmer:      Andrew Porokhnyak
****************************************************************************
*
\**************************************************************************/

#include <sys/types.h>
#include <sys/stat.h>
#include <dir.h>
#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <ctype.h>
#include <string.h>


#include "defines.h"
#include "readcfg.h"
#include "getw.h"
#include "tools.h"


static char *path;
static int line;

char CFG_NAME[MAXPATH]="pl-mail.ctl";   /*name of cfg file*/

char EXT_FILES[]=".?lo";                /*extn mask for files list*/
char EXT_MAIL[]=".?ut";                 /*extn mask for packed mail*/
char EXT_PKT[]=".pkt";                  /*extn mask for mail packet*/

char MY_INP[MAXPATH]="files";           /*my inbound*/
char MY_OUT[MAXPATH]="out";             /*my outbound*/
char MY_FLAG[MAXPATH]="toss.now";       /*my flag to toss*/
char UL_INP[MAXPATH]="files";           /*UL inbound*/
char UL_OUT[MAXPATH]="out";             /*UL outbound*/
char UL_FLAG[MAXPATH]="toss.now";       /*UL flag to toss*/

#ifdef zero
int my_z=-1;                            /**/
int my_n=-1;                            /**/
int my_f=-1;                            /**/
int my_p=0;                             /**/
int ul_z=-1;                            /**/
int ul_n=-1;                            /**/
int ul_f=-1;                            /**/
int ul_p=0;                             /**/
#endif

int conlog=4;
int loglevel=4;
char logpath[MAXPATH]="pl-mail.log";


int nAddr = 0;
FTN_ADDR *pAddr = 0;
FTN_ADDR my_addr;
FTN_ADDR ul_addr;


/***************************************************************************
***************************************************************************/
typedef struct _KEYWORD KEYWORD;
struct _KEYWORD {
    const char *key;
    void (*callback) (KEYWORD *key, char *s);
    void *var;
    int option1;
    int option2;
};


static void read_adr(KEYWORD *, char *);
static void read_int(KEYWORD *, char *);
static void read_string(KEYWORD *, char *);



#define DONT_CHECK 0x7fff

KEYWORD keywords[] = {
  {"log", read_string, logpath, 'f', 0},
  {"consolelog", read_int, &conlog, 0, DONT_CHECK},
  {"loglevel", read_int, &loglevel, 0, DONT_CHECK},
  {"address", read_adr, &my_addr, 0, 0},
  {"my_flag", read_string, MY_FLAG, 0, MAXPATH},
  {"inbound", read_string, MY_INP, 'd', 0},
  {"outbound", read_string, MY_OUT, 'd', 0},

  {"uplink", read_adr, &ul_addr, 0, 0},
  {"up_flag", read_string, UL_FLAG, 0, MAXPATH},
  {"up_inb", read_string, UL_INP, 'd', 0},
  {"up_out", read_string, UL_OUT, 'd', 0},

  {NULL, NULL, NULL, 0, 0}
};

#define TEST(var) if (!*var) Log(0, "%s: "#var" should be defined", path)


void readcfg (char *_path) {
    FILE *in;
    char buf[MAXCFGLINE + 1];
    char *w;

    line = 0;
    path = _path;

    if ((in = fopen (path, "r")) == 0)
        Log (0, "%s: %s", path, strerror (errno));

    while (!feof (in)) {
        if (!fgets (buf, sizeof (buf), in))
            break;
        ++line;

        if ((w = getword (buf, 1)) != 0) {
            int j;

            for (j = 0; keywords[j].key; ++j)
                if (!stricmp (keywords[j].key, w))
                    break;

            if (keywords[j].key) {
                keywords[j].callback (keywords + j, buf);
            } else {
                Log(0, "%s: %i: %s: unknown keyword", path, line, w);
            }
            free (w);
        }
    }
    fclose (in);

    if (MY_INP[strlen(MY_INP)-1]!='\\') strcat(MY_INP, "\\");
    if (MY_OUT[strlen(MY_OUT)-1]!='\\') strcat(MY_OUT, "\\");
    if (UL_INP[strlen(UL_INP)-1]!='\\') strcat(UL_INP, "\\");
    if (UL_OUT[strlen(UL_OUT)-1]!='\\') strcat(UL_OUT, "\\");

}

/*
 *  METHODS TO PROCESS KEYWORDS' ARGUMETS
 */

static void read_adr (KEYWORD *key, char *s) {
    int i;
    FTN_ADDR *target = (FTN_ADDR *) (key->var);
    char *stmp;
    char *w, *p;

    if ((w = getword (s, 2)) != 0 && getword (s, 3) == 0) {
        stmp=w;

        if ((p=strchr(stmp, ':')) == NULL)
            Log(0, "%s: %i: bad FIDO address `%s'", path, line, *stmp);
        *p='\0';
        target->zon=atoi(stmp);

        stmp=p+1;
        if ((p=strchr(stmp, '/')) == NULL)
            Log(0, "%s: %i: bad FIDO address `%s'", path, line, *stmp);
        *p='\0';
        target->net=atoi(stmp);

        stmp=p+1;
        if ((p=strchr(stmp, '.')) == NULL) {
            target->nod=atoi(stmp);
            target->pnt=0;
        } else {
            *p='\0';
            target->nod=atoi(stmp);
            stmp=p+1;
            target->pnt=atoi(stmp);
        }


        free (w);
        /*
        strncpy (target, w, key->option1 == 0 ? key->option2 : MAXPATH);
        free (w);

        if (key->option1 != 0) {
            w = target + strlen (target) - 1;
            while (w >= target && (*w == '/' || *w == '\\')) {
                if (key->option1 == 'f') {
                    Log(0, "%s: %i: unexpected `%c' at the end of filename",
                    path, line, *w);
                }
                *(w--) = 0;
            }
            if (key->option1 == 'd' && (stat (target, &sb) == -1 ||
                      !(sb.st_mode & S_IFDIR))) {
                Log (0,"%s: %i: %s: incorrect directory", path, line, target);
            }
        }
        */

    } else
        Log (0, "%s: %i: the syntax is incorrect", path, line);



#ifdef zero
    for (i = 1; (w = getword (s, i+1)) != 0; ++i) {
    pAddr = realloc (pAddr, sizeof (FTN_ADDR) * (nAddr + 1));

    assert(pAddr != 0);
    if (!parse_ftnaddress (w, pAddr + nAddr)) {
        Log (0, "%s: %i: %s: the address cannot be parsed", path, line, w);
    }
    if (!is4D (pAddr + nAddr)) {
        Log (0, "%s: %i: %s: must be at least a 4D address", path, line, w);
    }
    if (!pAddr[nAddr].domain[0]) {
      if (!pDomains)
	Log (0, "%s: %i: at least one domain must be defined first", path, line);
      strcpy (pAddr[nAddr].domain, get_def_domain ()->name);
    }
    ++nAddr;
    free (w);

  }
#endif

}


static void check_dir_path (char *s) {
    if (s) {
        char *w = s + strlen (s) - 1;

        while (w >= s && (*w == '/' || *w == '\\'))
            *(w--) = 0;
    }
}


/*
 * Read a string (key->option1 == 0)
 * or a directory name (key->option1 == 'd')
 * or a file name (key->option1 == 'f')
 */
static void read_string (KEYWORD *key, char *s) {

    struct stat sb;
    char *target = (char *) (key->var);
    char *w;

    if ((w = getword (s, 2)) != 0 && getword (s, 3) == 0) {
        strncpy (target, w, key->option1 == 0 ? key->option2 : MAXPATH);
        free (w);

        if (key->option1 != 0) {
            w = target + strlen (target) - 1;
            while (w >= target && (*w == '/' || *w == '\\')) {
                if (key->option1 == 'f') {
                    Log(0, "%s: %i: unexpected `%c' at the end of filename",
                    path, line, *w);
                }
                *(w--) = 0;
            }
            if (key->option1 == 'd' && (stat (target, &sb) == -1 ||
                      !(sb.st_mode & S_IFDIR))) {
                Log (0,"%s: %i: %s: incorrect directory", path, line, target);
            }
        }
    } else
        Log (0, "%s: %i: the syntax is incorrect", path, line);

} /*----------------------------------------------------------------------*/


static void read_int (KEYWORD *key, char *s) {

    int *target = (int *) (key->var);
    char *w;

    if ((w = getword (s, 2)) != 0 && getword (s, 3) == 0) {
        *target = atoi (w);
        if ((key->option1 != DONT_CHECK &&
                *target < key->option1) ||
            (key->option2 != DONT_CHECK &&
                *target > key->option2)) {
            Log (0, "%s: %i: %i: incorrect value", path, line, *target);
        }
        free (w);
    } else
        Log (0, "%s: %i: the syntax is incorrect", path, line);

} /*----------------------------------------------------------------------*/

/*** EOF READCFG.C ********************************************************/
