/**************************************************************************\
*   BC++ 1.01                                   Source: SESSION.C
*   Host platform:   DOS                        Code:   ASCII-7
*   Target platform: DOS                        Code:   ASCII-7
*   Project name:    PL-Mail: Local FTN-Mailer (BSO)
*   Last change:     22.12.97
*   Programmer:      Andrew Porokhnyak
****************************************************************************
*
\**************************************************************************/

#include <fcntl.h>
#include <io.h>
#include <dir.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <sys\stat.h>

#include "defines.h"
#include "tools.h"
#include "readcfg.h"
#include "session.h"

/* maximum tries for copying mail and files */
#define _max_tries 15

int transfer_mail(const char *sour, const char *name, const char *pdst);
int transfer_files(const char *nflo, const char *pdst, int *count);

/***************************************************************************
* Do FTN-send mail and files for system `adst' from `psrc' BSO-directory to
* `pdst' directory; create an empty flag file `nflag' for tosser if
* something was send
*/
int ftn_send(const char *psrc, const char *pdst, const char *nflag,
        FTN_ADDR adst) {

    int day;
    int result;
    char pathn[MAXPATH];
    char sour[MAXPATH], targ[MAXPATH];
    char fname[MAXPATH], stmp[MAXPATH];
    struct ffblk fbl;
    int done;
    int HAVE_MAIL=0;
    int HAVE_FILE=0;

    strcpy(sour, psrc);
    strcpy(pathn, psrc);

    /* check is destination point or node */
    if (adst.pnt == 0) {
        sprintf(fname, "%04x%04x", adst.net, adst.nod);
    } else {
        sprintf(fname, "%04x%04x\.pnt%s%08x", adst.net, adst.nod,
            PATH_DELIM, adst.pnt);
        sprintf(stmp, "%04x%04x\.pnt%s", adst.net, adst.nod, PATH_DELIM);
        strcat(pathn, stmp);
    }

    Log(5, "%s --> %s", sour, pdst);
    strcat(sour, fname);

    /* process mail packets (<addr>.?out) */
    strcpy(stmp, sour);
    strcat(stmp, EXT_MAIL);
    Log(7, "Source pkt mask: %s", stmp);
    done=findfirst(stmp, &fbl, 0);
    while (!done) {
        strcpy(targ, pathn);
        strcat(targ, fbl.ff_name);
        Log(8, "Sendable .?ut: %s", targ);
        Log(4, "mail  %s", targ);
        if (transfer_mail(targ, fbl.ff_name, pdst))
            Log(1, "ftn_send(): Not copied %s", targ);
        else
            HAVE_MAIL++;
        done=findnext(&fbl);
    }

    /* process file packets (<addr>.?lo) */
    strcpy(stmp, sour);
    strcat(stmp, EXT_FILES);
    Log(7, "Source flo mask: %s", stmp);
    done=findfirst(stmp, &fbl, 0);
    while (!done) {
        strcpy(targ, pathn);
        strcat(targ, fbl.ff_name);
        Log(8, "Sendable .?lo: %s", targ);
        transfer_files(targ, pdst, &HAVE_FILE);
//        HAVE_FILE++;
        done=findnext(&fbl);
    }

    Log(4, "Total: mail %i, file %i", HAVE_MAIL, HAVE_FILE);

    /* set flag for tosser */
    if (HAVE_MAIL || HAVE_FILE) {
        FILE *flag=fopen(nflag, "w");
        fclose(flag);
        Log(5, "Set tosser flag %s", nflag);
    }

    return 0;
} /*----------------------------------------------------------------------*/


/***************************************************************************
* Transfer mail packet `name' (full path `sour') to `pdst'
* Return: 0 - all OK; 1 - error while copying
* TODO: call idle for delaying
*/
int transfer_mail(const char *sour, const char *name, const char *pdst) {
    char dest[MAXPATH];
    int tries=0;

    strcpy(dest, pdst);
    strcat(dest, name);
    dest[strlen(dest)-strlen(EXT_MAIL)]=0;
    strcat(dest, EXT_PKT);

    while (fcopy(dest, sour, 'w')) {
        unsigned int i=strlen(dest)-strlen(EXT_MAIL)-1;
        dest[i]+=1;
        /*not tested ((dest[i]==58) ? dest[i]=97 : dest[i]++); */
        if (++tries>=_max_tries) return 1; /* exit */
    }

    Log(9, "%s --> %s", sour, dest);
    fdelete(sour);
    return 0;
} /*----------------------------------------------------------------------*/


int transfer_files(const char *nflo, const char *pdst, int *count) {
    FILE *flo;
    char stmp[MAXPATH];
    char dest[MAXPATH];
    char *p, *q;

    if ((flo=fopen(nflo, "r")) == NULL) {
        Log(1, "transfer_files(): Cannot open flo file %s", nflo);
        return 1;
    }
    if (!pdst) {
        Log(1, "transfer_files(): No destination path");
    }

    Log(9, ".?LO file %s opened", nflo);
    while (fgets(stmp, MAXPATH-1, flo)) {
        int tries=0;
        /* remove trailing line terminator */
        stmp[strlen(stmp)-1]='\0';
        p=strtok(stmp, " \t");
        p++;                            /*full source name*/

        q=p+strlen(p);
        while (*(--q) != PATH_DELIM[0]) ;
        strcpy(dest, pdst);
        strcat(dest, q+1);
        Log(9, "dest %s", dest);

        while (fcopy(dest, p, 'w') && (tries++<_max_tries)) {
            unsigned int i=strlen(dest)-1;
//            dest[i]+=1;
            if (dest[i]==58)
                dest[i]=97;
            else
                dest[i]+=1;
        }

        if (tries>=_max_tries)
            Log(1, "transfer_files(): Not copied %s", p);
        else {
            Log(4, "file  %s", p);
            Log(7, "copied to %s", dest);
            (*count)++;
            ftrunc(p);
        }

        if (feof(flo)) break;
    }

    fclose(flo);
    fdelete(nflo);
    return 0;
} /*----------------------------------------------------------------------*/

/*** EOF SESSION.C ********************************************************/
