
#ifdef zero
#include <sys/types.h>
#include "defines.h"
#endif

#include <sys/stat.h>
#include <errno.h>
#include <time.h>
#include <stdio.h>
#include <stdlib.h>
#include <fcntl.h>
#include <stdarg.h>
#include <string.h>
#include <ctype.h>
#include <io.h>
#include <dir.h>

#include "tools.h"
#include "readcfg.h"

/*  Lowercase the string
*/
char *strlower (char *s) {
  int i;

  for (i = 0; s[i]; ++i)
    s[i] = tolower (s[i]);
  return s;
}


void Log(int lev, char *s,...) {

    static int first_time = 1;
    char timebuf[40];
    char stmp[256];
    time_t t;
    struct tm *tm;
    va_list ap;
    /* chars to ident loglevel  0123456789 */
    static const char *marks = "!?+- *$#@.";
    char ch = (0 <= lev && lev < (int) strlen (marks)) ? marks[lev] : ' ';
    /* Loglevels are:
    *   0 (!) Fatal error, program aborted
    *   1 (?) Non-fatal error, program will continued
    *   2 (+) Most valuable info about proram run
    *   3 (-) Less valuable info about proram run
    *   4 ( ) Normal info
    *   5 (*) Redundance info
    *   6 ($) Important variables values
    *   7 (#) Small debug
    *   8 (@) Medium debug
    *   9 (.) Full debug
    */

    time(&t);
    tm = localtime(&t);

    strftime (timebuf, sizeof (timebuf), "%d %b %H:%M:%S", tm);
    va_start(ap, s);
    vsprintf(stmp, s, ap);
    va_end (ap);

    if (lev <= conlog) {
        printf("%c %s %s\n", ch, timebuf, stmp);
    }

    if (lev <= loglevel && *logpath) {
        FILE *logfile = 0;
        int i;
        for (i=0; logfile==0 && i<10; ++i) {
            logfile = fopen (logpath, "a");
        }
        if (logfile) {
            if (first_time) {
                fputc('\n', logfile);
                first_time = 0;
            }
            fprintf(logfile, "%c %s %s\n", ch, timebuf, stmp);
            fclose(logfile);
        } else {
            fprintf(stderr, "\a\nFatal: cannot open %s - %s\n", logpath,
            strerror(errno));
            exit(2);
        }
    }


#ifdef zero
    if (lev <= loglevel && *logpath) {
        FILE *logfile = 0;
        int i;

        for (i=0; logfile==0 && i<10; ++i) {
            logfile = fopen (logpath, "a");
        }
        if (logfile) {
            if (first_time) {
                fputc('\n', logfile);
                first_time = 0;
            }
            strftime (timebuf, sizeof (timebuf), "%d %b %H:%M:%S", tm);
            fprintf(logfile, "%c %s ", ch, timebuf);
            va_start(ap, s);
            vfprintf(logfile, s, ap);
            va_end (ap);
            fputc ('\n', logfile);
            fclose (logfile);
        }
        else {
            fprintf(stderr, "\a\nFatal: cannot open %s - %s\n", logpath,
            strerror(errno));
            exit(2);
        }
    }
#endif

    if (lev==0)
        exit(1);

} /*----------------------------------------------------------------------*/


/*
 * Remove/trucate a file, log this
 */
int fdelete(const char *path) {
    int rc;

    if ((rc = unlink (path)) == -1)
        Log(1, "error unlinking `%s': %s", path, strerror (errno));
    else
        Log(9, "unlinked `%s'", path);

    return rc;
}

int ftrunc(const char *path) {
    int h;

    if ((h = open (path, O_WRONLY | O_TRUNC)) == -1) {
        Log(1, "cannot truncate `%s': %s", path, strerror (errno));
        return -1;
    } else {
        Log(9, "truncated %s", path);
        close (h);
        return 0;
    }
}


/***************************************************************************
*  function fcopy() perform copying a file named `sour' to a file named
*  `dest'; when dest exist and mode is 'o', dest will be overwrited; when 'a'
*  -- append; when 'w' -- no action
*  return: 0 ok, 1 no sour, 2 dest exist, 3 read err, 4 write err
*/
#define BUFF_SIZE 16384
int fcopy(const char *dest, const char *sour, const char mode) {

    unsigned char *buff;
    int inphnd, outhnd;
    int cnt;
    int inpmod=O_BINARY | O_RDONLY;
    int outmod=O_BINARY | O_RDWR;
    struct ffblk fb;

    switch (mode) {
        case 'a':
            outmod|=O_APPEND;
            break;
        case 'o':
            outmod|=O_TRUNC;
            break;
        default:
            outmod|=O_CREAT;
    }

    if (findfirst(dest, &fb, 0)==0) {
        if (mode=='w') return 2;
    } else
        outmod|=O_CREAT;

    if ((inphnd=open(sour, inpmod))==-1) return 1;
    if ((outhnd=open(dest, outmod, S_IREAD | S_IWRITE))==-1) {
        close(outhnd);
        close(inphnd);
        return 7;
    }

    if ((buff=malloc(BUFF_SIZE))==NULL) {
        close(inphnd);
        close(outhnd);
        return 5;
    }
    while (!eof(inphnd)) {
        if ((cnt=read(inphnd, buff, BUFF_SIZE))==-1) {
            close(inphnd);
            close(outhnd);
            free(buff);
            return 3;
        }
        if ((write(outhnd, buff, cnt))==-1) {
            close(inphnd);
            close(outhnd);
            free(buff);
            return 4;
        }
    }
    close(inphnd);
    close(outhnd);
    free(buff);
    return 0;
}  /*----------------------------------------------------------------------*/



